@@ -16,7 +16,7 @@ import json
 import time
 from ast import literal_eval
 from collections import Counter
-from typing import TYPE_CHECKING, Dict, Iterable, List, Optional, cast
+from typing import TYPE_CHECKING, Dict, List, Optional, Sequence, Union, cast
 
 import qsharp  # type: ignore
 import qsharp.azure  # type: ignore
@@ -71,6 +71,7 @@ class AzureBackend(_QsharpBaseBackend):
 
     _supports_counts = True
     _supports_contextual_optimisation = True
+    _persistent_handles = True
 
     def __init__(
         self,
@@ -100,7 +101,7 @@ class AzureBackend(_QsharpBaseBackend):
         :raises RuntimeError: Azure authentication error
         :raises ValueError: Target not available
         """
-        super().__init__()
+        super().__init__(backend_name=target_name)
         self._MACHINE_DEBUG = machine_debug
         if not self._MACHINE_DEBUG:
             try:
@@ -170,8 +171,8 @@ class AzureBackend(_QsharpBaseBackend):
 
     def process_circuits(
         self,
-        circuits: Iterable[Circuit],
-        n_shots: Optional[int] = None,
+        circuits: Sequence[Circuit],
+        n_shots: Union[None, int, Sequence[Optional[int]]] = None,
         valid_check: bool = True,
         **kwargs: KwargTypes,
     ) -> List[ResultHandle]:
@@ -179,8 +180,12 @@ class AzureBackend(_QsharpBaseBackend):
         See :py:meth:`pytket.backends.Backend.process_circuits`.
         Supported kwargs: `postprocess`.
         """
-        if n_shots is None or n_shots < 1:
-            raise ValueError("Parameter n_shots is required for this backend")
+        circuits = list(circuits)
+        n_shots_list = Backend._get_n_shots_as_list(
+            n_shots,
+            len(circuits),
+            optional=False,
+        )
 
         if valid_check:
             self._check_all_circuits(circuits, nomeasure_warn=True)
@@ -188,7 +193,7 @@ class AzureBackend(_QsharpBaseBackend):
         postprocess = kwargs.get("postprocess", False)
 
         handles = []
-        for c in circuits:
+        for c, n_shots in zip(circuits, n_shots_list):
             if postprocess:
                 c0, ppcirc = prepare_circuit(c, allow_classical=False)
                 ppcirc_rep = ppcirc.to_dict()
@@ -212,6 +217,14 @@ class AzureBackend(_QsharpBaseBackend):
             self._cache[handle] = dict()
         return handles
 
+    def _update_cache_result(
+        self, handle: ResultHandle, result_dict: Dict[str, BackendResult]
+    ) -> None:
+        if handle in self._cache:
+            self._cache[handle].update(result_dict)
+        else:
+            self._cache[handle] = result_dict
+
     def circuit_status(self, handle: ResultHandle) -> CircuitStatus:
         self._check_handle_type(handle)
         jobid = cast(str, handle[0])
@@ -222,7 +235,7 @@ class AzureBackend(_QsharpBaseBackend):
         if self._MACHINE_DEBUG:
             n_bits = literal_eval(jobid[len(_DEBUG_HANDLE_PREFIX) :])
             empty_ar = OutcomeArray.from_ints([0] * n_shots, n_bits, big_endian=True)
-            self._cache[handle].update({"result": BackendResult(shots=empty_ar)})
+            self._update_cache_result(handle, {"result": BackendResult(shots=empty_ar)})
             statenum = StatusEnum.COMPLETED
         else:
             job = qsharp.azure.status(jobid)
@@ -231,8 +244,8 @@ class AzureBackend(_QsharpBaseBackend):
             message = repr(job)
             if statenum is StatusEnum.COMPLETED:
                 output = qsharp.azure.output(jobid)
-                self._cache[handle].update(
-                    {"result": _convert_result(output, n_shots, ppcirc=ppcirc)}
+                self._update_cache_result(
+                    handle, {"result": _convert_result(output, n_shots, ppcirc=ppcirc)}
                 )
         return CircuitStatus(statenum, message)
 
